<?php 
###########################################################
# PHP/XML/SVG File Menu Creation Script
# Adam S. Kirschner
# December 31, 2004
#
# Version: 2.0
# This script will read in from an XML file and this PHP script
# will output in SVG a file menu system.
#####################################################################################################
# Configuration
#####################################################################################################

# path of where the menu xml configuration file is
define( "FILENAME", "menu3.xml" );

# background color of the document
define( "BACKGROUND_COLOR", "#000066" );

# x and y coordinate of menu
define( "MENU_X", 10 );
define( "MENU_Y", 40 );

# width and height of menu
define( "MENU_WIDTH", 90 );
define( "MENU_HEIGHT", 20 );

# color of border around the menu
define( "RECT_STROKE", "#FFFFFF" );

# color of menu item
define( "MENU_ITEM_OFF", "#000055" );

# color of menu item when hovered
define( "MENU_ITEM_ON", "#0000DD" );

# font color
define( "FONT_COLOR", "#FFFFFF" );

# text that would appear before the content if the menu item has a submenu
define( "SUB_MENU_ITEM_PREFIX", "*" );

# text that would appear after the content if the menu item has a submenu
define( "SUB_MENU_ITEM_SUFFIX", "" );


#####################################################################################################

	# change content type to SVG
	header("Content-type: image/svg+xml"); 
	echo "<?xml version=\"1.0\" standalone=\"no\"?>\n"; 

	# load the xml file 
	$xml_data = simplexml_load_file( FILENAME );
	
	# start printing out the menu
	print_menu( $xml_data );

/************************************************************************
* Function: 		print_menu
* Purpose:		recursive function, prints out a menu
* Returns:		nothing
* Outputs: 		SVG menu code
* Parmeters: 
*
*	xml_data:	the entire XML menu (returned from simplexml_load_file)
*************************************************************************/
function print_menu( $xml_data ) {
	$menu_item_id = -1;
?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN" "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd">
<svg>
	<g id="document">
		<rect x="0" y="0" width="1000" height="1000" fill="<?=BACKGROUND_COLOR?>" />
			<text x="10" y="20" fill="white">This requires PHP5 (uses simplexml_load_file)</text>
			<a xlink:href="http://www.php.net/simplexml_load_file">
				<text x="270" y="20" fill="white" style="text-decoration: underline;">
					http://php.net/simplexml_load_file
				</text>
			</a>
	</g>

	<rect x="<?=MENU_X?>" y="<?=MENU_Y?>" height="<?=MENU_HEIGHT?>" width="<?=MENU_WIDTH * get_count( $xml_data )?>" style="stroke: <?=RECT_STROKE?>; fill: <?=MENU_ITEM_OFF?>; stroke-width: 1px;" />
<?php

	# go through each menu
	foreach( $xml_data->menu as $menu ) {

		# set up menu ids
		$menu_counts[] = array();
		$menu_item_id++;

		# x and y coordinate of the next menu
		$x = MENU_X + $menu_item_id * MENU_WIDTH;
		$y = MENU_Y;

		# trim the menu's text and the location, may have tabs around it from XML layout.
		$content = trim( $menu );
		$href = trim( $menu["href"] );

		echo "<g id=\"menu_$menu_item_id\">\n";

		# print the main menu item
		print_sub_menu_item( $x, $y, $content, $href, $menu_item_id, 0 );

		# start counting
		array_push( $menu_counts, get_count( $xml_data ) );

		# start printing sub menus
		if( isset( $menu->menuitem ) ) {

			# -=-=-=-=- BUG FIX 12/9/2004 @ 3:33pm -=-=-=-
			# Add Menu Count To general menu counts. 
			array_push( $menu_counts, get_count( $menu ) );
			# -=-=-=-=-=-=--= END BUG FIX -=-=-=-=-=-=-=-

			print_sub_menu( $menu_item_id, $menu_counts, 0, $x, $y + 1 + MENU_HEIGHT, $menu );
		}

		print "</g>\n";
		
	}
}

/************************************************************************
* Function: 		print_sub_menu
* Purpose:		recursive function, prints out a menu
* Returns:		nothing
* Outputs: 		SVG menu code
* Parmeters: 
*
*	menu_id: 	id number of what this menu is (counter added to be unique)
*	menu_counts:	the amount of items each generation of menus have
*	recursed:	the amount of times this function was recursed 
*			(used to set actions on grandchildren and children)
*	x:		x-coordinate of menu item
*	y: 		y-coordinate of menu item
*	menu:		the menu itself in an array
*************************************************************************/
function print_sub_menu( $menu_id, $menu_counts, $recursed, $x, $y, $menu ) {

	# -=-=-=-=- BUG FIX 12/9/2004 @ 3:33pm -=-=-=-
	$grandparent_menu_count = 3;
	$parent_menu_count = 2;
	# -=-=-=-=-=-=--= END BUG FIX -=-=-=-=-=-=-=-

	$height = MENU_HEIGHT * get_count( $menu );
	$width = MENU_WIDTH;
	$rect_stroke = RECT_STROKE;
	$menu_item_off = MENU_ITEM_OFF;
	$counter = 0;
	$parent_menu = substr( $menu_id, 0, -1 );
	$grandparent_menu = substr( $menu_id, 0, -2 );
	
	echo "<g id=\"items_$menu_id\" style=\"visibility: hidden;\">\n";
	echo "<set attributeName=\"visibility\" to=\"hidden\" begin=\"document.mouseover\" />\n";	
	if ( $recursed == 0 ) {
		for ( $k = 0; $k < $menu_counts[1]; $k++ ) {
			if ( $k != $menu_id ) {
				echo "\t<set attributeName=\"visibility\" to=\"hidden\" begin=\"menu_$k.mouseover\" />\n";
			}
		}

	}

	# If I have 2 or more submenus, I have to set actions for all 
	# the my siblings and myself so that if you hover over our grandfather, we will hide
	if ( $recursed > 1 ) {
		for( $loop = 0; $loop < $menu_counts[get_count($menu_counts) - $grandparent_menu_count]; $loop++ ) {
			echo "<set attributeName=\"visibility\" to=\"hidden\" begin=\"link_$grandparent_menu$loop.mouseover\" />\n";
		}
	}
	# fade in the menu once my parent is hovered on
	echo "<animate attributeName=\"fill-opacity\" from=\"0.0\" to=\"1.0\" dur=\"0.5s\" begin=\"link_$menu_id.mouseover\" />\n";
	echo "<set attributeName=\"visibility\" to=\"visibile\" begin=\"link_$menu_id.mouseover\" />\n";
	
	# If this has been recursed at all, then there are parents, otherwise there aren't
	if ( $recursed > 0 ) {		

		# I have to set all of my siblings to hide my submenu
		for( $loop = 0; $loop < $menu_counts[get_count($menu_counts) - $parent_menu_count]; $loop++ ) {
		
			#make sure to skip this menu, otherwise i'll have mouseover and mouseout events for hidden visibility
			if ( "$parent_menu$loop" != "$menu_id" ) {
				echo " <set attributeName=\"visibility\" to=\"hidden\" begin=\"link_$parent_menu$loop.mouseover\" />\n";
			}

		}
	}	
	# this is the rectangle that goes around all the menu items
	echo "<rect x=\"$x\" y=\"$y\" height=\"$height\" width=\"$width\" style=\"stroke: $rect_stroke; fill: $menu_item_off; stroke-width: 1px;\" />\n\n";

	# go through and print out each menu item, each menu item is in an array
	foreach ( $menu->menuitem as $menu_item ) {

		# the id of this menu, every menu is uniquly identified
		$menu_item_id = $menu_id . $counter++;

		# this item's content and link
		$content = trim( $menu_item );
		$href = trim( $menu_item["href"] );

		# print the actual menu item
		if ( isset( $menu_item->menuitem ) ) { 
			print_sub_menu_item( $x, $y, $content, $href, $menu_item_id, 1 );
		} else {
			print_menu_item( $x, $y, $content, $href, $menu_item_id );
		}


		# Detect if another submenu exists
		if( isset( $menu_item->menuitem ) ) {

			# Calculate the location of the submenu
			$new_x = $x + MENU_WIDTH + 1;
			$new_y = $y;

			# make an array of all the submenus, it gets used to hide THIS menu, 
			# not the recursed ones (unless if THIS menu is a  grandparent)
			array_push( $menu_counts, get_count($menu_item) );
			
			# Recurse, I have more menus to do.
			print_sub_menu( $menu_item_id, $menu_counts, $recursed + 1,$new_x, $new_y, $menu_item );

		}

		# get ready to print out next menu (if any)
		$y += MENU_HEIGHT;
	}

	echo "</g>\n";


}

/************************************************************************
* Function: 		print_menu_item
* Purpose:		prints out a menu item in SVG with default
*			actions such as standard hover affect
* Returns:		nothing
* Outputs: 		SVG menu item code
* Parmeters: 
*	x:		x-coordinate of menu item
*	y: 		y-coordinate of menu item
*	content:	caption, text of menu item
*	href:		link, location when clicked on
*	menu_id: 	id number of this menu item (should be unique)	
*************************************************************************/
function print_menu_item( $x, $y, $content, $href, $menu_id ) {
	
	$height = MENU_HEIGHT;
	$width = MENU_WIDTH;
	$menu_item_off = MENU_ITEM_OFF;
	$menu_item_on = MENU_ITEM_ON;
	$menu_item_off = MENU_ITEM_OFF;
	$font_color = FONT_COLOR;

	# Calculate the text location. This is based on a fixed size.
	# (X) 4 and (Y) 14 are adjustment coordinates that may affect 
	# font size. I didn't incorporate text font and size for this reason.
	$text_x = $x + 4;
	$text_y = $y + 14;
	
?>
<a xlink:href="<?=$href?>">
	<rect x="<?=$x?>" y="<?=$y?>" height="<?=$height?>" width="<?=$width?>" fill="<?=$menu_item_off?>" id="link_<?=$menu_id?>">
		<set attributeName="fill" to="<?=$menu_item_on?>" begin="link_<?=$menu_id?>.mouseover" />
		<set attributeName="fill" to="<?=$menu_item_off?>" begin="link_<?=$menu_id?>.mouseout" />
	</rect>
	<text x="<?=$text_x?>" y="<?=$text_y?>" fill="<?=$font_color?>" style="pointer-events: none;">
		<?=$content . "\n"?>
	</text>
</a>
<?php
}

/************************************************************************
* Function: 		print_sub_menu_item
* Purpose:		prints out a sub menu item in SVG with default
*			actions such as standard hover affect 
			Note: this does the same as print_sub_menu_item
			except there are 3 extra set lines that keep
			this item selected when a child has been selected
* Returns:		nothing
* Outputs: 		SVG sub menu item code
* Parmeters: 
*	x:		x-coordinate of menu item
*	y: 		y-coordinate of menu item
*	content:	caption, text of menu item
*	href:		link, location when clicked on
*	menu_id: 	id number of this menu item (should be unique)	
*	show_text:	this shows/hides the prefix/suffix if one is set
			(a main menu item does NOT show this extra text)
*************************************************************************/
function print_sub_menu_item( $x, $y, $content, $href, $menu_id, $show_text ) {
	
	$height = MENU_HEIGHT;
	$width = MENU_WIDTH;
	$menu_item_off = MENU_ITEM_OFF;
	$menu_item_on = MENU_ITEM_ON;
	$menu_item_off = MENU_ITEM_OFF;
	$font_color = FONT_COLOR;

	# check to see whether or not to show the extra text
	if ( $show_text ) {
		$content = SUB_MENU_ITEM_PREFIX . "$content" . SUB_MENU_ITEM_SUFFIX;
	}

	# Calculate the text location. This is based on a fixed size.
	# (X) 4 and (Y) 14 are adjustment coordinates that may affect 
	# font size. I didn't incorporate text font and size for this reason.
	$text_x = $x + 4;
	$text_y = $y + 14;
	
?>
<a xlink:href="<?=$href?>">
	<rect x="<?=$x?>" y="<?=$y?>" height="<?=$height?>" width="<?=$width?>" fill="<?=$menu_item_off?>" id="link_<?=$menu_id?>">
		<set attributeName="fill" to="<?=$menu_item_on?>" begin="link_<?=$menu_id?>.mouseover" />
		<set attributeName="fill" to="<?=$menu_item_off?>" begin="link_<?=$menu_id?>.mouseout" />
		<set attributeName="fill" to="<?=$menu_item_on?>" begin="items_<?=$menu_id?>.mouseover" />
		<set attributeName="fill" to="<?=$menu_item_off?>" begin="items_<?=$menu_id?>.mouseout" />
		<set attributeName="fill" to="<?=$menu_item_off?>" begin="document.mouseover" />
	</rect>
	<text x="<?=$text_x?>" y="<?=$text_y?>" fill="<?=$font_color?>" style="pointer-events: none;">
		<?=$content  . "\n"?>
	</text>
</a>
<?php
}

/************************************************************************
* Function: 		get_count
* Purpose:		counts the amount of items in $object
*			used to count the number of menu items in the
*			xml menu since count() doesn't work with objects
*			only arrays.
* Returns:		int, amount of items in $object
* Outputs:		nothing
* Parmeters: 
*	object:		the object to be counted
*************************************************************************/
function get_count( $object ) {
	$counter = 0;
	foreach( $object as $item ) {
		$counter++;
	}
	return $counter;
}


?>
</svg>
